#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" The production package is responsible for classes which describe various kinds of generators. These classes also provide production costing information which is used to economically allocate demand among committed units and calculate reserve quantities.The production package is responsible for classes which describe various kinds of generators. These classes also provide production costing information which is used to economically allocate demand among committed units and calculate reserve quantities.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CDPSM.IEC61970.Core import Equipment
from CDPSM.IEC61970.Domain import ActivePower



from enthought.traits.api import Instance, List, Property, Enum
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


GeneratorControlSource = Enum("onAGC", "unavailable", "plantControl", "offAGC")

#------------------------------------------------------------------------------
#  "GeneratingUnit" class:
#------------------------------------------------------------------------------

class GeneratingUnit(Equipment):
    """ A single or set of synchronous machines for converting mechanical power into alternating-current power. For example, individual machines within a set may be defined for scheduling purposes while a single control signal is derived for the set. In this case there would be a GeneratingUnit for each member of the set and an additional GeneratingUnit corresponding to the set.A single or set of synchronous machines for converting mechanical power into alternating-current power. For example, individual machines within a set may be defined for scheduling purposes while a single control signal is derived for the set. In this case there would be a GeneratingUnit for each member of the set and an additional GeneratingUnit corresponding to the set.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A synchronous machine may operate as a generator and as such becomes a member of a generating unitA synchronous machine may operate as a generator and as such becomes a member of a generating unit
    SynchronousMachines = List(Instance("CDPSM.IEC61970.Wires.SynchronousMachine"),
        desc="A synchronous machine may operate as a generator and as such becomes a member of a generating unitA synchronous machine may operate as a generator and as such becomes a member of a generating unit")

    # The net rated maximum capacity determined by subtracting the auxiliary power used to operate the internal plant machinery from the rated gross maximum capacityThe net rated maximum capacity determined by subtracting the auxiliary power used to operate the internal plant machinery from the rated gross maximum capacity
    ratedNetMaxP = ActivePower(desc="The net rated maximum capacity determined by subtracting the auxiliary power used to operate the internal plant machinery from the rated gross maximum capacityThe net rated maximum capacity determined by subtracting the auxiliary power used to operate the internal plant machinery from the rated gross maximum capacity")

    # The source of controls for a generating unit.The source of controls for a generating unit.
    genControlSource = GeneratorControlSource(desc="The source of controls for a generating unit.The source of controls for a generating unit.")

    # Default Initial active power  which is used to store a powerflow result for the initial active power for this unit in this network configurationDefault Initial active power  which is used to store a powerflow result for the initial active power for this unit in this network configuration
    initialP = ActivePower(desc="Default Initial active power  which is used to store a powerflow result for the initial active power for this unit in this network configurationDefault Initial active power  which is used to store a powerflow result for the initial active power for this unit in this network configuration")

    #--------------------------------------------------------------------------
    #  Begin "GeneratingUnit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "mRID", "description", "name", "localName", "aliasName", "normaIlyInService", "ratedNetMaxP", "genControlSource", "initialP",
                label="Attributes"),
            VGroup("Model", "GeoLocation", "PSRType", "EquipmentContainer", "SynchronousMachines",
                label="References"),
            dock="tab"),
        id="CDPSM.IEC61970.Generation.Production.GeneratingUnit",
        title="GeneratingUnit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "GeneratingUnit" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" The production package is responsible for classes which describe various kinds of generators. These classes also provide production costing information which is used to economically allocate demand among committed units and calculate reserve quantities.The production package is responsible for classes which describe various kinds of generators. These classes also provide production costing information which is used to economically allocate demand among committed units and calculate reserve quantities.
"""

from cdpsm.iec61970.core import Equipment

# <<< imports
# @generated
# >>> imports

ns_prefix = "cim"

ns_uri = "http://iec.ch/TC57/2009/CIM-schema-cim14#Package_Production"

class GeneratingUnit(Equipment):
    """ A single or set of synchronous machines for converting mechanical power into alternating-current power. For example, individual machines within a set may be defined for scheduling purposes while a single control signal is derived for the set. In this case there would be a GeneratingUnit for each member of the set and an additional GeneratingUnit corresponding to the set.A single or set of synchronous machines for converting mechanical power into alternating-current power. For example, individual machines within a set may be defined for scheduling purposes while a single control signal is derived for the set. In this case there would be a GeneratingUnit for each member of the set and an additional GeneratingUnit corresponding to the set.
    """
    # <<< generating_unit
    # @generated
    def __init__(self, rated_net_max_p=0.0, gen_control_source='on_agc', initial_p=0.0, synchronous_machines=None, **kw_args):
        """ Initialises a new 'GeneratingUnit' instance.
        """
        # The net rated maximum capacity determined by subtracting the auxiliary power used to operate the internal plant machinery from the rated gross maximum capacityThe net rated maximum capacity determined by subtracting the auxiliary power used to operate the internal plant machinery from the rated gross maximum capacity 
        self.rated_net_max_p = rated_net_max_p

        # The source of controls for a generating unit.The source of controls for a generating unit. Values are: "on_agc", "unavailable", "plant_control", "off_agc"
        self.gen_control_source = 'on_agc'

        # Default Initial active power  which is used to store a powerflow result for the initial active power for this unit in this network configurationDefault Initial active power  which is used to store a powerflow result for the initial active power for this unit in this network configuration 
        self.initial_p = initial_p


        self._synchronous_machines = []
        if synchronous_machines is not None:
            self.synchronous_machines = synchronous_machines
        else:
            self.synchronous_machines = []


        super(GeneratingUnit, self).__init__(**kw_args)
    # >>> generating_unit

    # <<< synchronous_machines
    # @generated
    def get_synchronous_machines(self):
        """ A synchronous machine may operate as a generator and as such becomes a member of a generating unitA synchronous machine may operate as a generator and as such becomes a member of a generating unit
        """
        return self._synchronous_machines

    def set_synchronous_machines(self, value):
        for x in self._synchronous_machines:
            x._generating_unit = None
        for y in value:
            y._generating_unit = self
        self._synchronous_machines = value

    synchronous_machines = property(get_synchronous_machines, set_synchronous_machines)

    def add_synchronous_machines(self, *synchronous_machines):
        for obj in synchronous_machines:
            obj._generating_unit = self
            self._synchronous_machines.append(obj)

    def remove_synchronous_machines(self, *synchronous_machines):
        for obj in synchronous_machines:
            obj._generating_unit = None
            self._synchronous_machines.remove(obj)
    # >>> synchronous_machines


    def __str__(self):
        """ Returns a string representation of the GeneratingUnit.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< generating_unit.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the GeneratingUnit.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "GeneratingUnit", self.uri)
        if format:
            indent += ' ' * depth

        for obj in self.synchronous_machines:
            s += '%s<%s:GeneratingUnit.synchronous_machines rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:GeneratingUnit.rated_net_max_p>%s</%s:GeneratingUnit.rated_net_max_p>' % \
            (indent, ns_prefix, self.rated_net_max_p, ns_prefix)
        s += '%s<%s:GeneratingUnit.gen_control_source>%s</%s:GeneratingUnit.gen_control_source>' % \
            (indent, ns_prefix, self.gen_control_source, ns_prefix)
        s += '%s<%s:GeneratingUnit.initial_p>%s</%s:GeneratingUnit.initial_p>' % \
            (indent, ns_prefix, self.initial_p, ns_prefix)
        if self.model is not None:
            s += '%s<%s:Element.model rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.model.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        if self.geo_location is not None:
            s += '%s<%s:PowerSystemResource.geo_location rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.geo_location.uri)
        if self.psrtype is not None:
            s += '%s<%s:PowerSystemResource.psrtype rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.psrtype.uri)
        if self.equipment_container is not None:
            s += '%s<%s:Equipment.equipment_container rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.equipment_container.uri)
        s += '%s<%s:Equipment.norma_ily_in_service>%s</%s:Equipment.norma_ily_in_service>' % \
            (indent, ns_prefix, self.norma_ily_in_service, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "GeneratingUnit")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> generating_unit.serialize


# <<< production
# @generated
# >>> production



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" This package is an extension of Assets package and contains the core information classes that support asset management and different network and work planning applications with specialized documentation classes describing assets of a particular product model made by a manufacturer. There are typically many instances of an asset associated with a single asset model. It also contains 'lightweight' *Info classes, which hold model attributes that can be referenced by not only Assets but also by ConductingEquipments.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CIM.IEC61968.Common import Document
from CIM.IEC61970.Core import IdentifiedObject
from CIM.IEC61970.Domain import Weight
from CIM.IEC61970.Wires import WindingConnection
from CIM.IEC61970.Domain import ApparentPower
from CIM.IEC61970.Domain import Voltage
from CIM.IEC61970.Domain import Resistance
from CIM.IEC61970.Domain import Length
from CIM.IEC61970.Domain import Temperature
from CIM.IEC61970.Domain import CurrentFlow
from CIM.IEC61970.Domain import KWActivePower
from CIM.IEC61970.Domain import Impedance
from CIM.IEC61970.Domain import PerCent
from CIM.IEC61970.Domain import AngleDegrees



from enthought.traits.api import Instance, List, Property, Enum, Str, Int, Bool
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------

# Kind of conductor usage.
ConductorUsageKind = Enum("secondary", "transmission", "other", "distribution", desc="Kind of conductor usage.")
# Kind of conductor material.
ConductorMaterialKind = Enum("aluminum", "steel", "other", "copper", "acsr", desc="Kind of conductor material.")
# Kind of cable outer jacket.
CableOuterJacketKind = Enum("linearLowDensityPolyethylene", "semiconducting", "none", "other", "pvc", "insulating", "polyethylene", desc="Kind of cable outer jacket.")
# Kind of cable construction.
CableConstructionKind = Enum("compacted", "sector", "segmental", "solid", "stranded", "other", "compressed", desc="Kind of cable construction.")
# Kind of cable shield material.
CableShieldMaterialKind = Enum("other", "aluminum", "steel", "lead", "copper", desc="Kind of cable shield material.")
# Usage for an asset model.
AssetModelUsageKind = Enum("distributionUnderground", "other", "streetlight", "customerSubstation", "unknown", "distributionOverhead", "substation", "transmission", desc="Usage for an asset model.")
# Kind of conductor insulation.
ConductorInsulationKind = Enum("highPressureFluidFilled", "lowCapacitanceRubber", "crosslinkedPolyethylene", "ozoneResistantRubber", "highMolecularWeightPolyethylene", "treeRetardantCrosslinkedPolyethylene", "oilPaper", "butyl", "beltedPilc", "rubber", "ethylenePropyleneRubber", "varnishedCambricCloth", "asbestosAndVarnishedCambric", "treeResistantHighMolecularWeightPolyethylene", "other", "siliconRubber", "unbeltedPilc", "varnishedDacronGlass", desc="Kind of conductor insulation.")
# Kind of corporate standard.
CorporateStandardKind = Enum("other", "underEvaluation", "experimental", "standard", desc="Kind of corporate standard.")

#------------------------------------------------------------------------------
#  "AssetModel" class:
#------------------------------------------------------------------------------

class AssetModel(Document):
    """ Documentation for a particular product model made by a manufacturer. There are typically many instances of an asset associated with a single asset model.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ErpInventoryCounts = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpInventoryCount"))

    # A type of asset may be satisified with many different types of asset models.
    TypeAsset = Instance("CIM.IEC61968.Informative.InfTypeAsset.TypeAsset",
        desc="A type of asset may be satisified with many different types of asset models.",
        transient=True,
        opposite="AssetModels",
        editor=InstanceEditor(name="_typeassets"))

    def _get_typeassets(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfTypeAsset.TypeAsset" ]
        else:
            return []

    _typeassets = Property(fget=_get_typeassets)

    AssetModelCatalogueItems = List(Instance("CIM.IEC61968.Informative.InfAssetModels.AssetModelCatalogueItem"))

    # Kind of corporate standard for this asset model.
    corporateStandardKind = CorporateStandardKind(desc="Kind of corporate standard for this asset model.")

    # Intended usage for this asset model.
    usageKind = AssetModelUsageKind(desc="Intended usage for this asset model.")

    # Manufacturer's model number.
    modelNumber = Str(desc="Manufacturer's model number.")

    # Total manufactured weight of asset.
    weightTotal = Weight(desc="Total manufactured weight of asset.")

    # Version number for product model, which indicates vintage of the product.
    modelVersion = Str(desc="Version number for product model, which indicates vintage of the product.")

    #--------------------------------------------------------------------------
    #  Begin "AssetModel" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "corporateStandardKind", "usageKind", "modelNumber", "weightTotal", "modelVersion",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "ErpInventoryCounts", "TypeAsset", "AssetModelCatalogueItems",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.AssetModel",
        title="AssetModel",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "AssetModel" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "DistributionWindingTest" class:
#------------------------------------------------------------------------------

class DistributionWindingTest(IdentifiedObject):
    """ Test results for one or more transformer windings. These may include short-circuit or open-circuit (excitation) tests. Short-circuit test results include load losses and leakage impedances. Open-circuit test results may include no-load losses, exciting current, phase shifts, and induced voltage. For three-phase windings, the excitation can be positive sequence (the default) or zero sequence.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Winding that voltage or current is applied to during the test.
    FromWinding = Instance("CIM.IEC61968.AssetModels.WindingInfo",
        desc="Winding that voltage or current is applied to during the test.",
        transient=True,
        opposite="WindingTests",
        editor=InstanceEditor(name="_windinginfos"))

    def _get_windinginfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.AssetModels.WindingInfo" ]
        else:
            return []

    _windinginfos = Property(fget=_get_windinginfos)

    # Tap step number for the 'from' winding of the test pair.
    fromTapStep = Int(desc="Tap step number for the 'from' winding of the test pair.")

    #--------------------------------------------------------------------------
    #  Begin "DistributionWindingTest" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "fromTapStep",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "FromWinding",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.DistributionWindingTest",
        title="DistributionWindingTest",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "DistributionWindingTest" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "WindingInfo" class:
#------------------------------------------------------------------------------

class WindingInfo(IdentifiedObject):
    """ Winding data.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Transformer data that this winding description is part of.
    TransformerInfo = Instance("CIM.IEC61968.AssetModels.TransformerInfo",
        desc="Transformer data that this winding description is part of.",
        transient=True,
        opposite="WindingInfos",
        editor=InstanceEditor(name="_transformerinfos"))

    def _get_transformerinfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.AssetModels.TransformerInfo" ]
        else:
            return []

    _transformerinfos = Property(fget=_get_transformerinfos)

    # All winding tests during which voltage or current was applied to this winding.
    WindingTests = List(Instance("CIM.IEC61968.AssetModels.DistributionWindingTest"),
        desc="All winding tests during which voltage or current was applied to this winding.")

    # All windings described by this winding data.
    Windings = List(Instance("CIM.IEC61968.WiresExt.DistributionTransformerWinding"),
        desc="All windings described by this winding data.")

    # Tap steps and induced voltage/angle measurements for tests in which this winding was not excited.
    ToWindingSpecs = List(Instance("CIM.IEC61968.AssetModels.ToWindingSpec"),
        desc="Tap steps and induced voltage/angle measurements for tests in which this winding was not excited.")

    # Kind of connection of this winding.
    connectionKind = WindingConnection(desc="Kind of connection of this winding.")

    # Apparent power that this winding can carry for a short period of time.
    shortTermS = ApparentPower(desc="Apparent power that this winding can carry for a short period of time.")

    # Basic insulation level voltage rating.
    insulationU = Voltage(desc="Basic insulation level voltage rating.")

    # Sequence number for this winding, corresponding to the winding's order in the TransformerBank.vectorGroup attribute. Highest voltage winding should be '1'.
    sequenceNumber = Int(desc="Sequence number for this winding, corresponding to the winding's order in the TransformerBank.vectorGroup attribute. Highest voltage winding should be '1'.")

    # Winding phase angle where 360 degrees are represented with clock hours, so the valid values are {0, ..., 11}. For example, to express winding code 'Dyn11', set attributes as follows: 'connectionKind' = Yn and 'phaseAngle' = 11.
    phaseAngle = Int(desc="Winding phase angle where 360 degrees are represented with clock hours, so the valid values are {0, ..., 11}. For example, to express winding code 'Dyn11', set attributes as follows: 'connectionKind' = Yn and 'phaseAngle' = 11.")

    # Apparent power that the winding can carry under emergency conditions.
    emergencyS = ApparentPower(desc="Apparent power that the winding can carry under emergency conditions.")

    # Rated voltage of this winding: phase-phase for three-phase windings, and either phase-phase or phase-neutral for single-phase windings.
    ratedU = Voltage(desc="Rated voltage of this winding: phase-phase for three-phase windings, and either phase-phase or phase-neutral for single-phase windings.")

    # DC resistance of this winding.
    r = Resistance(desc="DC resistance of this winding.")

    # Normal apparent power rating of this winding.
    ratedS = ApparentPower(desc="Normal apparent power rating of this winding.")

    #--------------------------------------------------------------------------
    #  Begin "WindingInfo" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "connectionKind", "shortTermS", "insulationU", "sequenceNumber", "phaseAngle", "emergencyS", "ratedU", "r", "ratedS",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "TransformerInfo", "WindingTests", "Windings", "ToWindingSpecs",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.WindingInfo",
        title="WindingInfo",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "WindingInfo" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ConductorInfo" class:
#------------------------------------------------------------------------------

class ConductorInfo(IdentifiedObject):
    """ Conductor data.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All conductor segments described by this conductor data.
    ConductorSegments = List(Instance("CIM.IEC61968.WiresExt.DistributionLineSegment"),
        desc="All conductor segments described by this conductor data.")

    ConductorAssetModel = Instance("CIM.IEC61968.Informative.InfAssetModels.ConductorAssetModel",
        transient=True,
        opposite="ConductorInfo",
        editor=InstanceEditor(name="_conductorassetmodels"))

    def _get_conductorassetmodels(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssetModels.ConductorAssetModel" ]
        else:
            return []

    _conductorassetmodels = Property(fget=_get_conductorassetmodels)

    # All wire arrangements (single wires) that make this conductor.
    WireArrangements = List(Instance("CIM.IEC61968.AssetModels.WireArrangement"),
        desc="All wire arrangements (single wires) that make this conductor.")

    # Usage of this conductor.
    usage = ConductorUsageKind(desc="Usage of this conductor.")

    # (if insulated conductor) Material used for insulation.
    insulationMaterial = ConductorInsulationKind(desc="(if insulated conductor) Material used for insulation.")

    # Number of phases (including neutral) to be retained. Any wires beyond this number should be reduced into the earth return.
    phaseCount = Int(desc="Number of phases (including neutral) to be retained. Any wires beyond this number should be reduced into the earth return.")

    # True if conductor is insulated.
    insulated = Bool(desc="True if conductor is insulated.")

    # (if insulated conductor) Thickness of the insulation.
    insulationThickness = Length(desc="(if insulated conductor) Thickness of the insulation.")

    #--------------------------------------------------------------------------
    #  Begin "ConductorInfo" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "usage", "insulationMaterial", "phaseCount", "insulated", "insulationThickness",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "ConductorSegments", "ConductorAssetModel", "WireArrangements",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.ConductorInfo",
        title="ConductorInfo",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ConductorInfo" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "WireArrangement" class:
#------------------------------------------------------------------------------

class WireArrangement(IdentifiedObject):
    """ Identification, spacing and configuration of the wires of a Conductor, with reference to their type.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Wire type used for this wire arrangement.
    WireType = Instance("CIM.IEC61968.AssetModels.WireType",
        desc="Wire type used for this wire arrangement.",
        transient=True,
        opposite="WireArrangements",
        editor=InstanceEditor(name="_wiretypes"))

    def _get_wiretypes(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.AssetModels.WireType" ]
        else:
            return []

    _wiretypes = Property(fget=_get_wiretypes)

    # Conductor data this wire arrangement belongs to.
    ConductorInfo = Instance("CIM.IEC61968.AssetModels.ConductorInfo",
        desc="Conductor data this wire arrangement belongs to.",
        transient=True,
        opposite="WireArrangements",
        editor=InstanceEditor(name="_conductorinfos"))

    def _get_conductorinfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.AssetModels.ConductorInfo" ]
        else:
            return []

    _conductorinfos = Property(fget=_get_conductorinfos)

    # Position number on the structure corresponding to this wire. For example, use 1..3 for phases and 4 for the neutral on a 3-phase structure. The individual phase assignments matter; for example, ABC will produce a different set of unbalanced line parameters, by phase, than BAC.
    position = Int(desc="Position number on the structure corresponding to this wire. For example, use 1..3 for phases and 4 for the neutral on a 3-phase structure. The individual phase assignments matter; for example, ABC will produce a different set of unbalanced line parameters, by phase, than BAC.")

    # Height above ground of the first wire.
    mountingPointY = Length(desc="Height above ground of the first wire.")

    # Signed horizontal distance from the first wire to a common reference point.
    mountingPointX = Length(desc="Signed horizontal distance from the first wire to a common reference point.")

    #--------------------------------------------------------------------------
    #  Begin "WireArrangement" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "position", "mountingPointY", "mountingPointX",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "WireType", "ConductorInfo",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.WireArrangement",
        title="WireArrangement",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "WireArrangement" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "WireType" class:
#------------------------------------------------------------------------------

class WireType(IdentifiedObject):
    """ Wire conductor (per IEEE specs). A specific type of wire or combination of wires, not insulated from each other, suitable for carrying electrical current.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All wire arrangements using this wire type.
    WireArrangements = List(Instance("CIM.IEC61968.AssetModels.WireArrangement"),
        desc="All wire arrangements using this wire type.")

    # All concentric neutral cables using this wire type.
    ConcentricNeutralCableInfos = List(Instance("CIM.IEC61968.AssetModels.ConcentricNeutralCableInfo"),
        desc="All concentric neutral cables using this wire type.")

    # Wire material.
    material = ConductorMaterialKind(desc="Wire material.")

    # Number of strands in the wire.
    strandCount = Int(desc="Number of strands in the wire.")

    # AC resistance per unit length of the conductor at 25 degrees C.
    rAC25 = Resistance(desc="AC resistance per unit length of the conductor at 25 degrees C.")

    # (if used) Number of strands in the steel core.
    coreStrandCount = Int(desc="(if used) Number of strands in the steel core.")

    # Current carrying capacity of the wire under stated thermal conditions.
    ratedCurrent = CurrentFlow(desc="Current carrying capacity of the wire under stated thermal conditions.")

    # Describes the wire guage or cross section (e.g., 4/0, #2, 336.5).
    sizeDescription = Str(desc="Describes the wire guage or cross section (e.g., 4/0, #2, 336.5).")

    # Geometric Mean Radius. If we replace the conductor by a thin walled tube of radius GMR, then its reactance is identical to the reactance of the actual conductor.
    gmr = Length(desc="Geometric Mean Radius. If we replace the conductor by a thin walled tube of radius GMR, then its reactance is identical to the reactance of the actual conductor.")

    # AC resistance per unit length of the conductor at 75 degrees C.
    rAC75 = Resistance(desc="AC resistance per unit length of the conductor at 75 degrees C.")

    # Outside radius of the wire.
    radius = Length(desc="Outside radius of the wire.")

    # DC resistance per unit length of the conductor at 20 degrees C.
    rDC20 = Resistance(desc="DC resistance per unit length of the conductor at 20 degrees C.")

    # AC resistance per unit length of the conductor at 50 degrees C.
    rAC50 = Resistance(desc="AC resistance per unit length of the conductor at 50 degrees C.")

    # (if there is a different core material) Radius of the central core.
    coreRadius = Length(desc="(if there is a different core material) Radius of the central core.")

    #--------------------------------------------------------------------------
    #  Begin "WireType" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "material", "strandCount", "rAC25", "coreStrandCount", "ratedCurrent", "sizeDescription", "gmr", "rAC75", "radius", "rDC20", "rAC50", "coreRadius",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "WireArrangements", "ConcentricNeutralCableInfos",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.WireType",
        title="WireType",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "WireType" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TransformerInfo" class:
#------------------------------------------------------------------------------

class TransformerInfo(IdentifiedObject):
    """ Set of transformer data, from an equipment library.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All transformers that can be described with this transformer data.
    Transformers = List(Instance("CIM.IEC61968.WiresExt.DistributionTransformer"),
        desc="All transformers that can be described with this transformer data.")

    TransformerAssets = List(Instance("CIM.IEC61968.Informative.InfAssets.TransformerAsset"))

    # Data for all the windings described by this transformer data.
    WindingInfos = List(Instance("CIM.IEC61968.AssetModels.WindingInfo"),
        desc="Data for all the windings described by this transformer data.")

    TransformerAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.TransformerAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "TransformerInfo" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "Transformers", "TransformerAssets", "WindingInfos", "TransformerAssetModels",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.TransformerInfo",
        title="TransformerInfo",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TransformerInfo" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ToWindingSpec" class:
#------------------------------------------------------------------------------

class ToWindingSpec(IdentifiedObject):
    """ For short-circuit tests, specifies the winding and tap for all short-circuited windings.  For open-circuit tests, specifies the winding, tap, induced voltage, and induced angle for any non-excited windings that were measured during the test. This won't apply if only the exciting current and no-load losses were measured.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All open-circuit tests in which this winding was measured.
    OpenCircuitTests = List(Instance("CIM.IEC61968.AssetModels.OpenCircuitTest"),
        desc="All open-circuit tests in which this winding was measured.")

    # All short-circuit tests in which this winding was short-circuited.
    ShortCircuitTests = List(Instance("CIM.IEC61968.AssetModels.ShortCircuitTest"),
        desc="All short-circuit tests in which this winding was short-circuited.")

    # Winding short-circuited in a short-circuit test, or measured for induced voltage and angle in an open-circuit test.
    ToWinding = Instance("CIM.IEC61968.AssetModels.WindingInfo",
        desc="Winding short-circuited in a short-circuit test, or measured for induced voltage and angle in an open-circuit test.",
        transient=True,
        opposite="ToWindingSpecs",
        editor=InstanceEditor(name="_windinginfos"))

    def _get_windinginfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.AssetModels.WindingInfo" ]
        else:
            return []

    _windinginfos = Property(fget=_get_windinginfos)

    # (if open-circuit test) Voltage measured at the open-circuited 'to' winding, with the 'from' winding set to the 'from' winding's rated voltage and all other windings open-circuited.
    voltage = Voltage(desc="(if open-circuit test) Voltage measured at the open-circuited 'to' winding, with the 'from' winding set to the 'from' winding's rated voltage and all other windings open-circuited.")

    # Tap step number for the 'to' winding of the test pair.
    toTapStep = Int(desc="Tap step number for the 'to' winding of the test pair.")

    # (if open-circuit test) Phase shift measured at the open-circuited 'to' winding, with the 'from' winding set to the 'from' winding's rated voltage and all other windings open-circuited.
    phaseShift = AngleDegrees(desc="(if open-circuit test) Phase shift measured at the open-circuited 'to' winding, with the 'from' winding set to the 'from' winding's rated voltage and all other windings open-circuited.")

    #--------------------------------------------------------------------------
    #  Begin "ToWindingSpec" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "voltage", "toTapStep", "phaseShift",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "OpenCircuitTests", "ShortCircuitTests", "ToWinding",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.ToWindingSpec",
        title="ToWindingSpec",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ToWindingSpec" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CableInfo" class:
#------------------------------------------------------------------------------

class CableInfo(ConductorInfo):
    """ Cable data.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Kind of construction of this cable.
    constructionKind = CableConstructionKind(desc="Kind of construction of this cable.")

    # Material of the shield.
    shieldMaterial = CableShieldMaterialKind(desc="Material of the shield.")

    # Kind of outer jacket of this cable.
    outerJacketKind = CableOuterJacketKind(desc="Kind of outer jacket of this cable.")

    # Diameter over the core, including any semi-con screen; should be the insulating layer's inside diameter.
    diameterOverCore = Length(desc="Diameter over the core, including any semi-con screen; should be the insulating layer's inside diameter.")

    # True if wire strands are extruded in a way to fill the voids in the cable.
    isStrandFill = Bool(desc="True if wire strands are extruded in a way to fill the voids in the cable.")

    # Diameter over the insulating layer, excluding outer screen.
    diameterOverInsulation = Length(desc="Diameter over the insulating layer, excluding outer screen.")

    # Diameter over the outermost jacketing layer.
    diameterOverJacket = Length(desc="Diameter over the outermost jacketing layer.")

    # Maximum nominal design operating temperature.
    nominalTemperature = Temperature(desc="Maximum nominal design operating temperature.")

    # True if sheath / shield is used as a neutral (i.e., bonded).
    sheathAsNeutral = Bool(desc="True if sheath / shield is used as a neutral (i.e., bonded).")

    # Diameter over the outer screen; should be the shield's inside diameter..
    diameterOverScreen = Length(desc="Diameter over the outer screen; should be the shield's inside diameter..")

    #--------------------------------------------------------------------------
    #  Begin "CableInfo" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "usage", "insulationMaterial", "phaseCount", "insulated", "insulationThickness", "constructionKind", "shieldMaterial", "outerJacketKind", "diameterOverCore", "isStrandFill", "diameterOverInsulation", "diameterOverJacket", "nominalTemperature", "sheathAsNeutral", "diameterOverScreen",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ConductorSegments", "ConductorAssetModel", "WireArrangements",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.CableInfo",
        title="CableInfo",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CableInfo" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ConcentricNeutralCableInfo" class:
#------------------------------------------------------------------------------

class ConcentricNeutralCableInfo(CableInfo):
    """ Concentric neutral cable data.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Wire type used for this concentric neutral cable.
    WireType = Instance("CIM.IEC61968.AssetModels.WireType",
        desc="Wire type used for this concentric neutral cable.",
        transient=True,
        opposite="ConcentricNeutralCableInfos",
        editor=InstanceEditor(name="_wiretypes"))

    def _get_wiretypes(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.AssetModels.WireType" ]
        else:
            return []

    _wiretypes = Property(fget=_get_wiretypes)

    # Number of concentric neutral strands.
    neutralStrandCount = Int(desc="Number of concentric neutral strands.")

    # Diameter over the concentric neutral strands.
    diameterOverNeutral = Length(desc="Diameter over the concentric neutral strands.")

    #--------------------------------------------------------------------------
    #  Begin "ConcentricNeutralCableInfo" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "usage", "insulationMaterial", "phaseCount", "insulated", "insulationThickness", "constructionKind", "shieldMaterial", "outerJacketKind", "diameterOverCore", "isStrandFill", "diameterOverInsulation", "diameterOverJacket", "nominalTemperature", "sheathAsNeutral", "diameterOverScreen", "neutralStrandCount", "diameterOverNeutral",
                label="Attributes", columns=2),
            VGroup("Parent", "ModelingAuthoritySet", "ConductorSegments", "ConductorAssetModel", "WireArrangements", "WireType",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.ConcentricNeutralCableInfo",
        title="ConcentricNeutralCableInfo",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ConcentricNeutralCableInfo" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ShortCircuitTest" class:
#------------------------------------------------------------------------------

class ShortCircuitTest(DistributionWindingTest):
    """ Short-circuit test results include load losses and leakage impedances. For three-phase windings, the excitation can be positive sequence (the default) or zero sequence. There must be at least one short-circuited ('to') winding.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All windings short-circuited during this test.
    ShortedWindingSpecs = List(Instance("CIM.IEC61968.AssetModels.ToWindingSpec"),
        desc="All windings short-circuited during this test.")

    # Load losses from a positive-sequence or single-phase short-circuit test.
    loadLoss = KWActivePower(desc="Load losses from a positive-sequence or single-phase short-circuit test.")

    # Load losses from a zero-sequence short-circuit test.
    loadLossZero = KWActivePower(desc="Load losses from a zero-sequence short-circuit test.")

    # Leakage impedance measured from a zero-sequence short-circuit test.
    leakageImpedanceZero = Impedance(desc="Leakage impedance measured from a zero-sequence short-circuit test.")

    # Leakage impedance measured from a positive-sequence or single-phase short-circuit test.
    leakageImpedance = Impedance(desc="Leakage impedance measured from a positive-sequence or single-phase short-circuit test.")

    #--------------------------------------------------------------------------
    #  Begin "ShortCircuitTest" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "fromTapStep", "loadLoss", "loadLossZero", "leakageImpedanceZero", "leakageImpedance",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "FromWinding", "ShortedWindingSpecs",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.ShortCircuitTest",
        title="ShortCircuitTest",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ShortCircuitTest" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "OverheadConductorInfo" class:
#------------------------------------------------------------------------------

class OverheadConductorInfo(ConductorInfo):
    """ Overhead conductor data.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # (if applicable) Insulation thickness of the neutral conductor.
    neutralInsulationThickness = Length(desc="(if applicable) Insulation thickness of the neutral conductor.")

    # Number of conductor strands in the symmetrical bundle (1-12).
    phaseConductorCount = Int(desc="Number of conductor strands in the symmetrical bundle (1-12).")

    # Distance between conductor strands in a symmetrical bundle.
    phaseConductorSpacing = Length(desc="Distance between conductor strands in a symmetrical bundle.")

    #--------------------------------------------------------------------------
    #  Begin "OverheadConductorInfo" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "usage", "insulationMaterial", "phaseCount", "insulated", "insulationThickness", "neutralInsulationThickness", "phaseConductorCount", "phaseConductorSpacing",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ConductorSegments", "ConductorAssetModel", "WireArrangements",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.OverheadConductorInfo",
        title="OverheadConductorInfo",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OverheadConductorInfo" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "OpenCircuitTest" class:
#------------------------------------------------------------------------------

class OpenCircuitTest(DistributionWindingTest):
    """ Open-circuit test results may include no-load losses, exciting current, phase shifts, and induced voltage. For three-phase windings, the excitation can be positive sequence (the default) or zero sequence. For induced voltage and phase shifts, use the associated ToWindingSpec class.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All other windings measured during this test.
    MeasuredWindingSpecs = List(Instance("CIM.IEC61968.AssetModels.ToWindingSpec"),
        desc="All other windings measured during this test.")

    # Losses measured from a positive-sequence or single-phase open-circuit (excitation) test.
    noLoadLoss = KWActivePower(desc="Losses measured from a positive-sequence or single-phase open-circuit (excitation) test.")

    # Exciting current measured from a zero-sequence open-circuit (excitation) test.
    excitingCurrentZero = PerCent(desc="Exciting current measured from a zero-sequence open-circuit (excitation) test.")

    # Exciting current measured from a positive-sequence or single-phase open-circuit (excitation) test.
    excitingCurrent = PerCent(desc="Exciting current measured from a positive-sequence or single-phase open-circuit (excitation) test.")

    # Losses measured from a zero-sequence open-circuit (excitation) test.
    noLoadLossZero = KWActivePower(desc="Losses measured from a zero-sequence open-circuit (excitation) test.")

    #--------------------------------------------------------------------------
    #  Begin "OpenCircuitTest" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "fromTapStep", "noLoadLoss", "excitingCurrentZero", "excitingCurrent", "noLoadLossZero",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "FromWinding", "MeasuredWindingSpecs",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.OpenCircuitTest",
        title="OpenCircuitTest",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OpenCircuitTest" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TapeShieldCableInfo" class:
#------------------------------------------------------------------------------

class TapeShieldCableInfo(CableInfo):
    """ Tape shield cable data.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Thickness of the tape shield, before wrapping.
    tapeThickness = Length(desc="Thickness of the tape shield, before wrapping.")

    # Percentage of the tape shield width that overlaps in each wrap, typically 10% to 25%.
    tapeLap = PerCent(desc="Percentage of the tape shield width that overlaps in each wrap, typically 10% to 25%.")

    #--------------------------------------------------------------------------
    #  Begin "TapeShieldCableInfo" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "usage", "insulationMaterial", "phaseCount", "insulated", "insulationThickness", "constructionKind", "shieldMaterial", "outerJacketKind", "diameterOverCore", "isStrandFill", "diameterOverInsulation", "diameterOverJacket", "nominalTemperature", "sheathAsNeutral", "diameterOverScreen", "tapeThickness", "tapeLap",
                label="Attributes", columns=2),
            VGroup("Parent", "ModelingAuthoritySet", "ConductorSegments", "ConductorAssetModel", "WireArrangements",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.TapeShieldCableInfo",
        title="TapeShieldCableInfo",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TapeShieldCableInfo" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "EndDeviceModel" class:
#------------------------------------------------------------------------------

class EndDeviceModel(AssetModel):
    """ Documentation for particular end device product model made by a manufacturer.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    EndDeviceTypeAsset = Instance("CIM.IEC61968.Informative.InfTypeAsset.EndDeviceTypeAsset",
        transient=True,
        opposite="EndDeviceModels",
        editor=InstanceEditor(name="_enddevicetypeassets"))

    def _get_enddevicetypeassets(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfTypeAsset.EndDeviceTypeAsset" ]
        else:
            return []

    _enddevicetypeassets = Property(fget=_get_enddevicetypeassets)

    # All end device assets being of this model.
    EndDeviceAssets = List(Instance("CIM.IEC61968.Metering.EndDeviceAsset"),
        desc="All end device assets being of this model.")

    #--------------------------------------------------------------------------
    #  Begin "EndDeviceModel" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "corporateStandardKind", "usageKind", "modelNumber", "weightTotal", "modelVersion",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "ErpInventoryCounts", "TypeAsset", "AssetModelCatalogueItems", "EndDeviceTypeAsset", "EndDeviceAssets",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.AssetModels.EndDeviceModel",
        title="EndDeviceModel",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EndDeviceModel" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" State variables for analysis solutions such as powerflow.
"""

from cim import Element

# <<< imports
# @generated
# >>> imports

ns_prefix = "cim.statevariables"

ns_uri = "http://iec.ch/TC57/2009/CIM-schema-cim14#StateVariables"

class StateVariable(Element):
    """ An abstract class for state variables.
    """
    pass
    # <<< state_variable
    # @generated
    def __init__(self, **kw_args):
        """ Initialises a new 'StateVariable' instance.
        """


        super(StateVariable, self).__init__(**kw_args)
    # >>> state_variable


    def __str__(self):
        """ Returns a string representation of the StateVariable.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< state_variable.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the StateVariable.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "StateVariable", self.uri)
        if format:
            indent += ' ' * depth

        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "StateVariable")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> state_variable.serialize


class SvInjection(StateVariable):
    """ Injectixon state variable.
    """
    # <<< sv_injection
    # @generated
    def __init__(self, p_net_injection=0.0, q_net_injection=0.0, topological_node=None, **kw_args):
        """ Initialises a new 'SvInjection' instance.
        """
        # The activive power injected into the bus at this location. 
        self.p_net_injection = p_net_injection

        # The activive power injected into the bus at this location. 
        self.q_net_injection = q_net_injection


        self._topological_node = None
        self.topological_node = topological_node


        super(SvInjection, self).__init__(**kw_args)
    # >>> sv_injection

    # <<< topological_node
    # @generated
    def get_topological_node(self):
        """ The topological node associated with the state injection.
        """
        return self._topological_node

    def set_topological_node(self, value):
        if self._topological_node is not None:
            self._topological_node._sv_injection = None

        self._topological_node = value
        if self._topological_node is not None:
            self._topological_node._sv_injection = self

    topological_node = property(get_topological_node, set_topological_node)
    # >>> topological_node


    def __str__(self):
        """ Returns a string representation of the SvInjection.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< sv_injection.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the SvInjection.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "SvInjection", self.uri)
        if format:
            indent += ' ' * depth

        if self.topological_node is not None:
            s += '%s<%s:SvInjection.topological_node rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.topological_node.uri)
        s += '%s<%s:SvInjection.p_net_injection>%s</%s:SvInjection.p_net_injection>' % \
            (indent, ns_prefix, self.p_net_injection, ns_prefix)
        s += '%s<%s:SvInjection.q_net_injection>%s</%s:SvInjection.q_net_injection>' % \
            (indent, ns_prefix, self.q_net_injection, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "SvInjection")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> sv_injection.serialize


class SvPowerFlow(StateVariable):
    """ State variable for power flow.
    """
    # <<< sv_power_flow
    # @generated
    def __init__(self, p=0.0, q=0.0, terminal=None, **kw_args):
        """ Initialises a new 'SvPowerFlow' instance.
        """
        # The active power flow into the terminal. 
        self.p = p

        # The reactive power flow into the terminal. 
        self.q = q


        self._terminal = None
        self.terminal = terminal


        super(SvPowerFlow, self).__init__(**kw_args)
    # >>> sv_power_flow

    # <<< terminal
    # @generated
    def get_terminal(self):
        """ The terminal associated with the power flow state.
        """
        return self._terminal

    def set_terminal(self, value):
        if self._terminal is not None:
            self._terminal._sv_power_flow = None

        self._terminal = value
        if self._terminal is not None:
            self._terminal._sv_power_flow = self

    terminal = property(get_terminal, set_terminal)
    # >>> terminal


    def __str__(self):
        """ Returns a string representation of the SvPowerFlow.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< sv_power_flow.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the SvPowerFlow.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "SvPowerFlow", self.uri)
        if format:
            indent += ' ' * depth

        if self.terminal is not None:
            s += '%s<%s:SvPowerFlow.terminal rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.terminal.uri)
        s += '%s<%s:SvPowerFlow.p>%s</%s:SvPowerFlow.p>' % \
            (indent, ns_prefix, self.p, ns_prefix)
        s += '%s<%s:SvPowerFlow.q>%s</%s:SvPowerFlow.q>' % \
            (indent, ns_prefix, self.q, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "SvPowerFlow")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> sv_power_flow.serialize


class SvStatus(StateVariable):
    """ State variable for status.
    """
    # <<< sv_status
    # @generated
    def __init__(self, in_service=False, conducting_equipment=None, **kw_args):
        """ Initialises a new 'SvStatus' instance.
        """
        # The in service status as a result of topology processing. 
        self.in_service = in_service


        self._conducting_equipment = None
        self.conducting_equipment = conducting_equipment


        super(SvStatus, self).__init__(**kw_args)
    # >>> sv_status

    # <<< conducting_equipment
    # @generated
    def get_conducting_equipment(self):
        """ The conducting equipment associated with the status state.
        """
        return self._conducting_equipment

    def set_conducting_equipment(self, value):
        if self._conducting_equipment is not None:
            self._conducting_equipment._sv_status = None

        self._conducting_equipment = value
        if self._conducting_equipment is not None:
            self._conducting_equipment._sv_status = self

    conducting_equipment = property(get_conducting_equipment, set_conducting_equipment)
    # >>> conducting_equipment


    def __str__(self):
        """ Returns a string representation of the SvStatus.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< sv_status.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the SvStatus.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "SvStatus", self.uri)
        if format:
            indent += ' ' * depth

        if self.conducting_equipment is not None:
            s += '%s<%s:SvStatus.conducting_equipment rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.conducting_equipment.uri)
        s += '%s<%s:SvStatus.in_service>%s</%s:SvStatus.in_service>' % \
            (indent, ns_prefix, self.in_service, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "SvStatus")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> sv_status.serialize


class SvVoltage(StateVariable):
    """ State variable for voltage.
    """
    # <<< sv_voltage
    # @generated
    def __init__(self, v=0.0, angle=0.0, topological_node=None, **kw_args):
        """ Initialises a new 'SvVoltage' instance.
        """
        # The voltage magnitude of the topological node. 
        self.v = v

        # The voltage angle in radians of the topological node. 
        self.angle = angle


        self._topological_node = None
        self.topological_node = topological_node


        super(SvVoltage, self).__init__(**kw_args)
    # >>> sv_voltage

    # <<< topological_node
    # @generated
    def get_topological_node(self):
        """ The topological node associated with the voltage state.
        """
        return self._topological_node

    def set_topological_node(self, value):
        if self._topological_node is not None:
            self._topological_node._sv_voltage = None

        self._topological_node = value
        if self._topological_node is not None:
            self._topological_node._sv_voltage = self

    topological_node = property(get_topological_node, set_topological_node)
    # >>> topological_node


    def __str__(self):
        """ Returns a string representation of the SvVoltage.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< sv_voltage.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the SvVoltage.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "SvVoltage", self.uri)
        if format:
            indent += ' ' * depth

        if self.topological_node is not None:
            s += '%s<%s:SvVoltage.topological_node rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.topological_node.uri)
        s += '%s<%s:SvVoltage.v>%s</%s:SvVoltage.v>' % \
            (indent, ns_prefix, self.v, ns_prefix)
        s += '%s<%s:SvVoltage.angle>%s</%s:SvVoltage.angle>' % \
            (indent, ns_prefix, self.angle, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "SvVoltage")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> sv_voltage.serialize


class SvTapStep(StateVariable):
    """ State variable for transformer tap step.     This class is to be used for taps of LTC (load tap changing) transformers, not fixed tap transformers.  Normally a profile specifies only one of the attributes 'position'or 'tapRatio'.
    """
    # <<< sv_tap_step
    # @generated
    def __init__(self, position=0, continuous_position=0.0, tap_changer=None, **kw_args):
        """ Initialises a new 'SvTapStep' instance.
        """
        # The integer tap position. 
        self.position = position

        # The floating point tap position. 
        self.continuous_position = continuous_position


        self._tap_changer = None
        self.tap_changer = tap_changer


        super(SvTapStep, self).__init__(**kw_args)
    # >>> sv_tap_step

    # <<< tap_changer
    # @generated
    def get_tap_changer(self):
        """ The tap changer associated with the tap step state.
        """
        return self._tap_changer

    def set_tap_changer(self, value):
        if self._tap_changer is not None:
            self._tap_changer._sv_tap_step = None

        self._tap_changer = value
        if self._tap_changer is not None:
            self._tap_changer._sv_tap_step = self

    tap_changer = property(get_tap_changer, set_tap_changer)
    # >>> tap_changer


    def __str__(self):
        """ Returns a string representation of the SvTapStep.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< sv_tap_step.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the SvTapStep.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "SvTapStep", self.uri)
        if format:
            indent += ' ' * depth

        if self.tap_changer is not None:
            s += '%s<%s:SvTapStep.tap_changer rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.tap_changer.uri)
        s += '%s<%s:SvTapStep.position>%s</%s:SvTapStep.position>' % \
            (indent, ns_prefix, self.position, ns_prefix)
        s += '%s<%s:SvTapStep.continuous_position>%s</%s:SvTapStep.continuous_position>' % \
            (indent, ns_prefix, self.continuous_position, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "SvTapStep")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> sv_tap_step.serialize


class SvShortCircuit(StateVariable):
    """ State variable for short circuit.
    """
    # <<< sv_short_circuit
    # @generated
    def __init__(self, x_per_r=0.0, r0_per_r=0.0, s_short_circuit=0.0, x0_per_x=0.0, topological_node=None, **kw_args):
        """ Initialises a new 'SvShortCircuit' instance.
        """
        # Ratio of positive sequence reactance per postive sequence resistance. 
        self.x_per_r = x_per_r

        # The ratio of zero sequence resistance to positive sequence resistance. 
        self.r0_per_r = r0_per_r

        # The short circuit apparent power drawn at this node when faulted. 
        self.s_short_circuit = s_short_circuit

        # The ratio of zero sequence reactance per positive sequence reactance. 
        self.x0_per_x = x0_per_x


        self._topological_node = None
        self.topological_node = topological_node


        super(SvShortCircuit, self).__init__(**kw_args)
    # >>> sv_short_circuit

    # <<< topological_node
    # @generated
    def get_topological_node(self):
        """ The topological node associated with the short circuit state.
        """
        return self._topological_node

    def set_topological_node(self, value):
        if self._topological_node is not None:
            self._topological_node._sv_short_circuit = None

        self._topological_node = value
        if self._topological_node is not None:
            self._topological_node._sv_short_circuit = self

    topological_node = property(get_topological_node, set_topological_node)
    # >>> topological_node


    def __str__(self):
        """ Returns a string representation of the SvShortCircuit.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< sv_short_circuit.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the SvShortCircuit.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "SvShortCircuit", self.uri)
        if format:
            indent += ' ' * depth

        if self.topological_node is not None:
            s += '%s<%s:SvShortCircuit.topological_node rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.topological_node.uri)
        s += '%s<%s:SvShortCircuit.x_per_r>%s</%s:SvShortCircuit.x_per_r>' % \
            (indent, ns_prefix, self.x_per_r, ns_prefix)
        s += '%s<%s:SvShortCircuit.r0_per_r>%s</%s:SvShortCircuit.r0_per_r>' % \
            (indent, ns_prefix, self.r0_per_r, ns_prefix)
        s += '%s<%s:SvShortCircuit.s_short_circuit>%s</%s:SvShortCircuit.s_short_circuit>' % \
            (indent, ns_prefix, self.s_short_circuit, ns_prefix)
        s += '%s<%s:SvShortCircuit.x0_per_x>%s</%s:SvShortCircuit.x0_per_x>' % \
            (indent, ns_prefix, self.x0_per_x, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "SvShortCircuit")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> sv_short_circuit.serialize


class SvShuntCompensatorSections(StateVariable):
    """ State variable for the number of sections in service for a shunt compensator.
    """
    # <<< sv_shunt_compensator_sections
    # @generated
    def __init__(self, sections=0, continuous_sections=0.0, shunt_compensator=None, **kw_args):
        """ Initialises a new 'SvShuntCompensatorSections' instance.
        """
        # The number of sections in service. 
        self.sections = sections

        # The number of sections in service as a continous variable. 
        self.continuous_sections = continuous_sections


        self._shunt_compensator = None
        self.shunt_compensator = shunt_compensator


        super(SvShuntCompensatorSections, self).__init__(**kw_args)
    # >>> sv_shunt_compensator_sections

    # <<< shunt_compensator
    # @generated
    def get_shunt_compensator(self):
        """ The shunt compensator for which the state applies.
        """
        return self._shunt_compensator

    def set_shunt_compensator(self, value):
        if self._shunt_compensator is not None:
            self._shunt_compensator._sv_shunt_compensator_sections = None

        self._shunt_compensator = value
        if self._shunt_compensator is not None:
            self._shunt_compensator._sv_shunt_compensator_sections = self

    shunt_compensator = property(get_shunt_compensator, set_shunt_compensator)
    # >>> shunt_compensator


    def __str__(self):
        """ Returns a string representation of the SvShuntCompensatorSections.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< sv_shunt_compensator_sections.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the SvShuntCompensatorSections.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "SvShuntCompensatorSections", self.uri)
        if format:
            indent += ' ' * depth

        if self.shunt_compensator is not None:
            s += '%s<%s:SvShuntCompensatorSections.shunt_compensator rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.shunt_compensator.uri)
        s += '%s<%s:SvShuntCompensatorSections.sections>%s</%s:SvShuntCompensatorSections.sections>' % \
            (indent, ns_prefix, self.sections, ns_prefix)
        s += '%s<%s:SvShuntCompensatorSections.continuous_sections>%s</%s:SvShuntCompensatorSections.continuous_sections>' % \
            (indent, ns_prefix, self.continuous_sections, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "SvShuntCompensatorSections")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> sv_shunt_compensator_sections.serialize


# <<< state_variables
# @generated
# >>> state_variables



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" This package is responsible for modeling the energy consumers and the system load as curves and associated curve data. Special circumstances that may affect the load, such as seasons and daytypes, are also included here.  This information is used by Load Forecasting and Load Management.This package is responsible for modeling the energy consumers and the system load as curves and associated curve data. Special circumstances that may affect the load, such as seasons and daytypes, are also included here.  This information is used by Load Forecasting and Load Management.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CPSM.IEC61970.Wires import EnergyConsumer
from CPSM.IEC61970.Core import IdentifiedObject
from CPSM import Element
from CPSM.IEC61970.Core import RegularIntervalSchedule



from enthought.traits.api import Instance, List, Property, Enum, Date, Float
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


SeasonName = Enum("fall", "winter", "spring", "summer")

#------------------------------------------------------------------------------
#  "NonConformLoad" class:
#------------------------------------------------------------------------------

class NonConformLoad(EnergyConsumer):
    """ NonConformLoad represent loads that do not follow a daily load change pattern and changes are not correlated with the daily load change pattern.NonConformLoad represent loads that do not follow a daily load change pattern and changes are not correlated with the daily load change pattern.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Group of this ConformLoad.Group of this ConformLoad.
    LoadGroup = Instance("CPSM.IEC61970.LoadModel.NonConformLoadGroup", allow_none=False,
        desc="Group of this ConformLoad.Group of this ConformLoad.",
        transient=True,
        opposite="EnergyConsumers",
        editor=InstanceEditor(name="_nonconformloadgroups"))

    def _get_nonconformloadgroups(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.LoadModel.NonConformLoadGroup" ]
        else:
            return []

    _nonconformloadgroups = Property(fget=_get_nonconformloadgroups)

    #--------------------------------------------------------------------------
    #  Begin "NonConformLoad" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "qfixedPct", "pfixed", "qfixed", "pfixedPct",
                label="Attributes"),
            VGroup("Model", "Contains_Measurements", "MemberOf_EquipmentContainer", "OperationalLimitSet", "BaseVoltage", "Terminals", "LoadResponse", "LoadGroup",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.NonConformLoad",
        title="NonConformLoad",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "NonConformLoad" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "DayType" class:
#------------------------------------------------------------------------------

class DayType(IdentifiedObject):
    """ Group of similar days, e.g., Mon/Tue/Wed, Thu/Fri, Sat/Sun, Holiday1, Holiday2Group of similar days, e.g., Mon/Tue/Wed, Thu/Fri, Sat/Sun, Holiday1, Holiday2
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Schedules that use this DayType.Schedules that use this DayType.
    SeasonDayTypeSchedules = List(Instance("CPSM.IEC61970.LoadModel.SeasonDayTypeSchedule"),
        desc="Schedules that use this DayType.Schedules that use this DayType.")

    #--------------------------------------------------------------------------
    #  Begin "DayType" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name",
                label="Attributes"),
            VGroup("Model", "SeasonDayTypeSchedules",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.DayType",
        title="DayType",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "DayType" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Season" class:
#------------------------------------------------------------------------------

class Season(Element):
    """ A specified time period of the year, e.g., Spring, Summer, Fall, WinterA specified time period of the year, e.g., Spring, Summer, Fall, Winter
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Schedules that use this Season.Schedules that use this Season.
    SeasonDayTypeSchedules = List(Instance("CPSM.IEC61970.LoadModel.SeasonDayTypeSchedule"),
        desc="Schedules that use this Season.Schedules that use this Season.")

    # Date season endsDate season ends
    endDate = Date(desc="Date season endsDate season ends")

    # Date season startsDate season starts
    startDate = Date(desc="Date season startsDate season starts")

    # Name of the SeasonName of the Season
    name = SeasonName(desc="Name of the SeasonName of the Season")

    #--------------------------------------------------------------------------
    #  Begin "Season" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "endDate", "startDate", "name",
                label="Attributes"),
            VGroup("Model", "SeasonDayTypeSchedules",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.Season",
        title="Season",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Season" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "StationSupply" class:
#------------------------------------------------------------------------------

class StationSupply(EnergyConsumer):
    """ Station supply with load derived from the station output.Station supply with load derived from the station output.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    pass
    #--------------------------------------------------------------------------
    #  Begin "StationSupply" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "qfixedPct", "pfixed", "qfixed", "pfixedPct",
                label="Attributes"),
            VGroup("Model", "Contains_Measurements", "MemberOf_EquipmentContainer", "OperationalLimitSet", "BaseVoltage", "Terminals", "LoadResponse",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.StationSupply",
        title="StationSupply",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "StationSupply" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SeasonDayTypeSchedule" class:
#------------------------------------------------------------------------------

class SeasonDayTypeSchedule(RegularIntervalSchedule):
    """ The schedule specialize RegularIntervalSchedule with type curve data for a specific type of day and season. This means that curves of this type cover a 24 hour period.The schedule specialize RegularIntervalSchedule with type curve data for a specific type of day and season. This means that curves of this type cover a 24 hour period.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # DayType for the Schedule.DayType for the Schedule.
    DayType = Instance("CPSM.IEC61970.LoadModel.DayType", allow_none=False,
        desc="DayType for the Schedule.DayType for the Schedule.",
        transient=True,
        opposite="SeasonDayTypeSchedules",
        editor=InstanceEditor(name="_daytypes"))

    def _get_daytypes(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.LoadModel.DayType" ]
        else:
            return []

    _daytypes = Property(fget=_get_daytypes)

    # Season for the Schedule.Season for the Schedule.
    Season = Instance("CPSM.IEC61970.LoadModel.Season", allow_none=False,
        desc="Season for the Schedule.Season for the Schedule.",
        transient=True,
        opposite="SeasonDayTypeSchedules",
        editor=InstanceEditor(name="_seasons"))

    def _get_seasons(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.LoadModel.Season" ]
        else:
            return []

    _seasons = Property(fget=_get_seasons)

    #--------------------------------------------------------------------------
    #  Begin "SeasonDayTypeSchedule" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "startTime", "value1Unit", "value2Unit", "endTime", "timeStep",
                label="Attributes"),
            VGroup("Model", "TimePoints", "DayType", "Season",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.SeasonDayTypeSchedule",
        title="SeasonDayTypeSchedule",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SeasonDayTypeSchedule" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "LoadGroup" class:
#------------------------------------------------------------------------------

class LoadGroup(IdentifiedObject):
    """ The class is the third level in a hierarchical structure for grouping of loads for the purpose of load flow load scaling.The class is the third level in a hierarchical structure for grouping of loads for the purpose of load flow load scaling.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The SubLoadArea where the Loadgroup belongs.The SubLoadArea where the Loadgroup belongs.
    SubLoadArea = Instance("CPSM.IEC61970.LoadModel.SubLoadArea", allow_none=False,
        desc="The SubLoadArea where the Loadgroup belongs.The SubLoadArea where the Loadgroup belongs.",
        transient=True,
        opposite="LoadGroups",
        editor=InstanceEditor(name="_subloadareas"))

    def _get_subloadareas(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.LoadModel.SubLoadArea" ]
        else:
            return []

    _subloadareas = Property(fget=_get_subloadareas)

    #--------------------------------------------------------------------------
    #  Begin "LoadGroup" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name",
                label="Attributes"),
            VGroup("Model", "SubLoadArea",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.LoadGroup",
        title="LoadGroup",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "LoadGroup" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "EnergyArea" class:
#------------------------------------------------------------------------------

class EnergyArea(IdentifiedObject):
    """ The class describes an area having energy production or consumption. The class is the basis for further specialization.The class describes an area having energy production or consumption. The class is the basis for further specialization.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The control area specification that is used for the load forecast.The control area specification that is used for the load forecast.
    ControlArea = Instance("CPSM.IEC61970.ControlArea.ControlArea",
        desc="The control area specification that is used for the load forecast.The control area specification that is used for the load forecast.",
        transient=True,
        opposite="EnergyArea",
        editor=InstanceEditor(name="_controlareas"))

    def _get_controlareas(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.ControlArea.ControlArea" ]
        else:
            return []

    _controlareas = Property(fget=_get_controlareas)

    #--------------------------------------------------------------------------
    #  Begin "EnergyArea" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name",
                label="Attributes"),
            VGroup("Model", "ControlArea",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.EnergyArea",
        title="EnergyArea",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EnergyArea" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "LoadResponseCharacteristic" class:
#------------------------------------------------------------------------------

class LoadResponseCharacteristic(IdentifiedObject):
    """ Models the characteristic response of the load demand due to to changes in system conditions such as voltage and frequency. This is not related to demand response.Models the characteristic response of the load demand due to to changes in system conditions such as voltage and frequency. This is not related to demand response.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The set of loads that have the response characteristics.The set of loads that have the response characteristics.
    EnergyConsumer = List(Instance("CPSM.IEC61970.Wires.EnergyConsumer"),
        desc="The set of loads that have the response characteristics.The set of loads that have the response characteristics.")

    # Exponent of per unit frequency effecting active powerExponent of per unit frequency effecting active power
    pFrequencyExponent = Float(desc="Exponent of per unit frequency effecting active powerExponent of per unit frequency effecting active power")

    # Exponent of per unit voltage effecting reactive power.   This model used only when 'useExponentModel' is true.Exponent of per unit voltage effecting reactive power.   This model used only when 'useExponentModel' is true.
    qVoltageExponent = Float(desc="Exponent of per unit voltage effecting reactive power.   This model used only when 'useExponentModel' is true.Exponent of per unit voltage effecting reactive power.   This model used only when 'useExponentModel' is true.")

    # Exponent of per unit frequency effecting reactive powerExponent of per unit frequency effecting reactive power
    qFrequencyExponent = Float(desc="Exponent of per unit frequency effecting reactive powerExponent of per unit frequency effecting reactive power")

    # Exponent of per unit voltage effecting real power.   This model used only when 'useExponentModel' is true.Exponent of per unit voltage effecting real power.   This model used only when 'useExponentModel' is true.
    pVoltageExponent = Float(desc="Exponent of per unit voltage effecting real power.   This model used only when 'useExponentModel' is true.Exponent of per unit voltage effecting real power.   This model used only when 'useExponentModel' is true.")

    #--------------------------------------------------------------------------
    #  Begin "LoadResponseCharacteristic" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "pFrequencyExponent", "qVoltageExponent", "qFrequencyExponent", "pVoltageExponent",
                label="Attributes"),
            VGroup("Model", "EnergyConsumer",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.LoadResponseCharacteristic",
        title="LoadResponseCharacteristic",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "LoadResponseCharacteristic" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ConformLoad" class:
#------------------------------------------------------------------------------

class ConformLoad(EnergyConsumer):
    """ ConformLoad represent loads that follow a daily load change pattern where the pattern can be used to scale the load with a system load.ConformLoad represent loads that follow a daily load change pattern where the pattern can be used to scale the load with a system load.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Group of this ConformLoad.Group of this ConformLoad.
    LoadGroup = Instance("CPSM.IEC61970.LoadModel.ConformLoadGroup", allow_none=False,
        desc="Group of this ConformLoad.Group of this ConformLoad.",
        transient=True,
        opposite="EnergyConsumers",
        editor=InstanceEditor(name="_conformloadgroups"))

    def _get_conformloadgroups(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.LoadModel.ConformLoadGroup" ]
        else:
            return []

    _conformloadgroups = Property(fget=_get_conformloadgroups)

    #--------------------------------------------------------------------------
    #  Begin "ConformLoad" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "qfixedPct", "pfixed", "qfixed", "pfixedPct",
                label="Attributes"),
            VGroup("Model", "Contains_Measurements", "MemberOf_EquipmentContainer", "OperationalLimitSet", "BaseVoltage", "Terminals", "LoadResponse", "LoadGroup",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.ConformLoad",
        title="ConformLoad",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ConformLoad" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "NonConformLoadGroup" class:
#------------------------------------------------------------------------------

class NonConformLoadGroup(LoadGroup):
    """ Loads that do not follow a daily and seasonal load variation pattern.Loads that do not follow a daily and seasonal load variation pattern.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The NonConformLoadSchedules in the NonConformLoadGroup.The NonConformLoadSchedules in the NonConformLoadGroup.
    NonConformLoadSchedules = List(Instance("CPSM.IEC61970.LoadModel.NonConformLoadSchedule"),
        desc="The NonConformLoadSchedules in the NonConformLoadGroup.The NonConformLoadSchedules in the NonConformLoadGroup.")

    # Conform loads assigned to this ConformLoadGroup.Conform loads assigned to this ConformLoadGroup.
    EnergyConsumers = List(Instance("CPSM.IEC61970.LoadModel.NonConformLoad"),
        desc="Conform loads assigned to this ConformLoadGroup.Conform loads assigned to this ConformLoadGroup.")

    #--------------------------------------------------------------------------
    #  Begin "NonConformLoadGroup" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name",
                label="Attributes"),
            VGroup("Model", "SubLoadArea", "NonConformLoadSchedules", "EnergyConsumers",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.NonConformLoadGroup",
        title="NonConformLoadGroup",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "NonConformLoadGroup" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ConformLoadSchedule" class:
#------------------------------------------------------------------------------

class ConformLoadSchedule(SeasonDayTypeSchedule):
    """ A curve of load  versus time (X-axis) showing the active power values (Y1-axis) and reactive power (Y2-axis) for each unit of the period covered. This curve represents a typical pattern of load over the time period for a given day type and season.A curve of load  versus time (X-axis) showing the active power values (Y1-axis) and reactive power (Y2-axis) for each unit of the period covered. This curve represents a typical pattern of load over the time period for a given day type and season.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The ConformLoadGroup where the ConformLoadSchedule belongs.The ConformLoadGroup where the ConformLoadSchedule belongs.
    ConformLoadGroup = Instance("CPSM.IEC61970.LoadModel.ConformLoadGroup", allow_none=False,
        desc="The ConformLoadGroup where the ConformLoadSchedule belongs.The ConformLoadGroup where the ConformLoadSchedule belongs.",
        transient=True,
        opposite="ConformLoadSchedules",
        editor=InstanceEditor(name="_conformloadgroups"))

    def _get_conformloadgroups(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.LoadModel.ConformLoadGroup" ]
        else:
            return []

    _conformloadgroups = Property(fget=_get_conformloadgroups)

    #--------------------------------------------------------------------------
    #  Begin "ConformLoadSchedule" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "startTime", "value1Unit", "value2Unit", "endTime", "timeStep",
                label="Attributes"),
            VGroup("Model", "TimePoints", "DayType", "Season", "ConformLoadGroup",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.ConformLoadSchedule",
        title="ConformLoadSchedule",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ConformLoadSchedule" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CustomerLoad" class:
#------------------------------------------------------------------------------

class CustomerLoad(ConformLoad):
    """ A meter for measuring customer energy consumption. The typeName attribute indicates the type of customer meter.A meter for measuring customer energy consumption. The typeName attribute indicates the type of customer meter.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    pass
    #--------------------------------------------------------------------------
    #  Begin "CustomerLoad" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "qfixedPct", "pfixed", "qfixed", "pfixedPct",
                label="Attributes"),
            VGroup("Model", "Contains_Measurements", "MemberOf_EquipmentContainer", "OperationalLimitSet", "BaseVoltage", "Terminals", "LoadResponse", "LoadGroup",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.CustomerLoad",
        title="CustomerLoad",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CustomerLoad" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Load" class:
#------------------------------------------------------------------------------

class Load(ConformLoad):
    """ A generic equivalent for an energy consumer on a transmission or distribution voltage level. It may be under load management and also has cold load pick up characteristics.A generic equivalent for an energy consumer on a transmission or distribution voltage level. It may be under load management and also has cold load pick up characteristics.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    pass
    #--------------------------------------------------------------------------
    #  Begin "Load" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "qfixedPct", "pfixed", "qfixed", "pfixedPct",
                label="Attributes"),
            VGroup("Model", "Contains_Measurements", "MemberOf_EquipmentContainer", "OperationalLimitSet", "BaseVoltage", "Terminals", "LoadResponse", "LoadGroup",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.Load",
        title="Load",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Load" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ConformLoadGroup" class:
#------------------------------------------------------------------------------

class ConformLoadGroup(LoadGroup):
    """ Load that follows a daily and seasonal load variation pattern.Load that follows a daily and seasonal load variation pattern.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Conform loads assigned to this ConformLoadGroup.Conform loads assigned to this ConformLoadGroup.
    EnergyConsumers = List(Instance("CPSM.IEC61970.LoadModel.ConformLoad"),
        desc="Conform loads assigned to this ConformLoadGroup.Conform loads assigned to this ConformLoadGroup.")

    # The ConformLoadSchedules in the ConformLoadGroup.The ConformLoadSchedules in the ConformLoadGroup.
    ConformLoadSchedules = List(Instance("CPSM.IEC61970.LoadModel.ConformLoadSchedule"),
        desc="The ConformLoadSchedules in the ConformLoadGroup.The ConformLoadSchedules in the ConformLoadGroup.")

    #--------------------------------------------------------------------------
    #  Begin "ConformLoadGroup" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name",
                label="Attributes"),
            VGroup("Model", "SubLoadArea", "EnergyConsumers", "ConformLoadSchedules",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.ConformLoadGroup",
        title="ConformLoadGroup",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ConformLoadGroup" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "LoadArea" class:
#------------------------------------------------------------------------------

class LoadArea(EnergyArea):
    """ The class is the root or first level in a hierarchical structure for grouping of loads for the purpose of load flow load scaling.The class is the root or first level in a hierarchical structure for grouping of loads for the purpose of load flow load scaling.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The SubLoadAreas in the LoadArea.The SubLoadAreas in the LoadArea.
    SubLoadAreas = List(Instance("CPSM.IEC61970.LoadModel.SubLoadArea"),
        desc="The SubLoadAreas in the LoadArea.The SubLoadAreas in the LoadArea.")

    #--------------------------------------------------------------------------
    #  Begin "LoadArea" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name",
                label="Attributes"),
            VGroup("Model", "ControlArea", "SubLoadAreas",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.LoadArea",
        title="LoadArea",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "LoadArea" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SubLoadArea" class:
#------------------------------------------------------------------------------

class SubLoadArea(EnergyArea):
    """ The class is the second level in a hierarchical structure for grouping of loads for the purpose of load flow load scaling.The class is the second level in a hierarchical structure for grouping of loads for the purpose of load flow load scaling.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The Loadgroups in the SubLoadArea.The Loadgroups in the SubLoadArea.
    LoadGroups = List(Instance("CPSM.IEC61970.LoadModel.LoadGroup"),
        desc="The Loadgroups in the SubLoadArea.The Loadgroups in the SubLoadArea.")

    # The LoadArea where the SubLoadArea belongs.The LoadArea where the SubLoadArea belongs.
    LoadArea = Instance("CPSM.IEC61970.LoadModel.LoadArea", allow_none=False,
        desc="The LoadArea where the SubLoadArea belongs.The LoadArea where the SubLoadArea belongs.",
        transient=True,
        opposite="SubLoadAreas",
        editor=InstanceEditor(name="_loadareas"))

    def _get_loadareas(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.LoadModel.LoadArea" ]
        else:
            return []

    _loadareas = Property(fget=_get_loadareas)

    #--------------------------------------------------------------------------
    #  Begin "SubLoadArea" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name",
                label="Attributes"),
            VGroup("Model", "ControlArea", "LoadGroups", "LoadArea",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.SubLoadArea",
        title="SubLoadArea",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SubLoadArea" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "NonConformLoadSchedule" class:
#------------------------------------------------------------------------------

class NonConformLoadSchedule(SeasonDayTypeSchedule):
    """ An active power (Y1-axis) and reactive power (Y2-axis) schedule (curves) versus time (X-axis) for non-conforming loads, e.g., large industrial load or power station service (where modeled)An active power (Y1-axis) and reactive power (Y2-axis) schedule (curves) versus time (X-axis) for non-conforming loads, e.g., large industrial load or power station service (where modeled)
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The NonConformLoadGroup where the NonConformLoadSchedule belongs.The NonConformLoadGroup where the NonConformLoadSchedule belongs.
    NonConformLoadGroup = Instance("CPSM.IEC61970.LoadModel.NonConformLoadGroup", allow_none=False,
        desc="The NonConformLoadGroup where the NonConformLoadSchedule belongs.The NonConformLoadGroup where the NonConformLoadSchedule belongs.",
        transient=True,
        opposite="NonConformLoadSchedules",
        editor=InstanceEditor(name="_nonconformloadgroups"))

    def _get_nonconformloadgroups(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.LoadModel.NonConformLoadGroup" ]
        else:
            return []

    _nonconformloadgroups = Property(fget=_get_nonconformloadgroups)

    #--------------------------------------------------------------------------
    #  Begin "NonConformLoadSchedule" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "startTime", "value1Unit", "value2Unit", "endTime", "timeStep",
                label="Attributes"),
            VGroup("Model", "TimePoints", "DayType", "Season", "NonConformLoadGroup",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.NonConformLoadSchedule",
        title="NonConformLoadSchedule",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "NonConformLoadSchedule" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "InductionMotorLoad" class:
#------------------------------------------------------------------------------

class InductionMotorLoad(NonConformLoad):
    """ Large three phase induction motor load. The typeName attribute indicates the type of induction motor (1 = wound rotor) (2 = squirrel cage).Large three phase induction motor load. The typeName attribute indicates the type of induction motor (1 = wound rotor) (2 = squirrel cage).
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    pass
    #--------------------------------------------------------------------------
    #  Begin "InductionMotorLoad" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "qfixedPct", "pfixed", "qfixed", "pfixedPct",
                label="Attributes"),
            VGroup("Model", "Contains_Measurements", "MemberOf_EquipmentContainer", "OperationalLimitSet", "BaseVoltage", "Terminals", "LoadResponse", "LoadGroup",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.LoadModel.InductionMotorLoad",
        title="InductionMotorLoad",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "InductionMotorLoad" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


""" Defines a map of class names to their module.
"""

cdpsm_pkg_map = {
    "Element": "cdpsm",
    "Model": "cdpsm",
    "DistributionTransformerWinding": "cdpsm.iec61968.wires_ext",
    "DistributionLineSegment": "cdpsm.iec61968.wires_ext",
    "WindingPiImpedance": "cdpsm.iec61968.wires_ext",
    "DistributionTapChanger": "cdpsm.iec61968.wires_ext",
    "PerLengthSequenceImpedance": "cdpsm.iec61968.wires_ext",
    "TransformerBank": "cdpsm.iec61968.wires_ext",
    "PerLengthPhaseImpedance": "cdpsm.iec61968.wires_ext",
    "DistributionTransformer": "cdpsm.iec61968.wires_ext",
    "PhaseImpedanceData": "cdpsm.iec61968.wires_ext",
    "TransformerInfo": "cdpsm.iec61968.asset_models",
    "ToWindingSpec": "cdpsm.iec61968.asset_models",
    "WireArrangement": "cdpsm.iec61968.asset_models",
    "CableInfo": "cdpsm.iec61968.asset_models",
    "OpenCircuitTest": "cdpsm.iec61968.asset_models",
    "ConcentricNeutralCableInfo": "cdpsm.iec61968.asset_models",
    "ConductorInfo": "cdpsm.iec61968.asset_models",
    "DistributionWindingTest": "cdpsm.iec61968.asset_models",
    "WireType": "cdpsm.iec61968.asset_models",
    "WindingInfo": "cdpsm.iec61968.asset_models",
    "OverheadConductorInfo": "cdpsm.iec61968.asset_models",
    "TapeShieldCableInfo": "cdpsm.iec61968.asset_models",
    "ShortCircuitTest": "cdpsm.iec61968.asset_models",
    "GeoLocation": "cdpsm.iec61968.common",
    "Location": "cdpsm.iec61968.common",
    "PositionPoint": "cdpsm.iec61968.common",
    "IEC61970CIMVersion": "cdpsm.iec61970",
    "BusbarSection": "cdpsm.iec61970.wires",
    "LoadBreakSwitch": "cdpsm.iec61970.wires",
    "TapChanger": "cdpsm.iec61970.wires",
    "Fuse": "cdpsm.iec61970.wires",
    "Junction": "cdpsm.iec61970.wires",
    "ACLineSegment": "cdpsm.iec61970.wires",
    "Disconnector": "cdpsm.iec61970.wires",
    "EnergySource": "cdpsm.iec61970.wires",
    "SynchronousMachine": "cdpsm.iec61970.wires",
    "RatioTapChanger": "cdpsm.iec61970.wires",
    "EnergyConsumer": "cdpsm.iec61970.wires",
    "Switch": "cdpsm.iec61970.wires",
    "Line": "cdpsm.iec61970.wires",
    "ShuntCompensator": "cdpsm.iec61970.wires",
    "Breaker": "cdpsm.iec61970.wires",
    "Conductor": "cdpsm.iec61970.wires",
    "ConnectivityNodeContainer": "cdpsm.iec61970.core",
    "VoltageLevel": "cdpsm.iec61970.core",
    "Bay": "cdpsm.iec61970.core",
    "Equipment": "cdpsm.iec61970.core",
    "BaseVoltage": "cdpsm.iec61970.core",
    "PSRType": "cdpsm.iec61970.core",
    "EquipmentContainer": "cdpsm.iec61970.core",
    "IdentifiedObject": "cdpsm.iec61970.core",
    "Substation": "cdpsm.iec61970.core",
    "ConductingEquipment": "cdpsm.iec61970.core",
    "SubGeographicalRegion": "cdpsm.iec61970.core",
    "Terminal": "cdpsm.iec61970.core",
    "GeographicalRegion": "cdpsm.iec61970.core",
    "PowerSystemResource": "cdpsm.iec61970.core",
    "GeneratingUnit": "cdpsm.iec61970.generation.production",
    "LoadResponseCharacteristic": "cdpsm.iec61970.load_model",
    "SvTapStep": "cdpsm.iec61970.state_variables",
    "ConnectivityNode": "cdpsm.iec61970.topology",
}
